#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <cs50.h>

typedef uint8_t BYTE;

#define BLOCK 512


int main(int argc, char *argv[])
{
    // Declare buffer array
    BYTE buffer[BLOCK];

    // Counter for JPEG names
    int counter_JPEGs = 0;
    char filename [8];
    FILE *outfile = NULL;
    bool fileopen = false;

    // Check if exactly 1 commandline argument was entered
    if (argc != 2)
    {
        printf("Usage: ./recover memorycard\n");
        return 1;
    }

    // Open input file (memory card) to read
    FILE *input = fopen(argv[1], "r");
    if (input == NULL)
    {
        printf("Could not open %s.\n", argv[1]);
        return 1;
    }

    // As long we don't reach the end of the document (when the returned size is less than 512) - read 512 bytes
    // while statement simultaneously executes read first block of bytes into buffer
    while (fread(buffer, BLOCK, 1, input) == 1)
    {
        // Now check first 4 bytes in the buffer to see if this is the start of a new jpeg
        if (buffer[0] == 0xff && buffer[1] == 0xd8 && buffer[2] == 0xff && (buffer[3] & 0xf0) == 0xe0)
        {
            // If this is the very first jpeg- only open a new one
            if (counter_JPEGs == 0)
            {
                // Set filename according to jpeg counter
                sprintf(filename, "%03i.jpg", counter_JPEGs);

                // printf("Current filename: %s\n", filename);

                // Open new file with the previously given filename
                outfile = fopen(filename, "w");

                fileopen = true;

                // Write current buffer into the new jpeg
                fwrite(buffer, BLOCK, 1, outfile);

                // Increment counter for next JPEG
                counter_JPEGs++;
            }

            // only else if!! Otherwise first if is executed and then second if is also true!
            else if (counter_JPEGs > 0)
            {
                // printf("Counter is at %i, so closing current file and opening a new one.\n", counter_JPEGs);  // DELETE LATER

                // Close previous file
                fclose(outfile);
                fileopen = false;

                // Set filename according to current jpeg counter (was incremented already after last jpeg was opened)
                sprintf(filename, "%03i.jpg", counter_JPEGs);

                // Open new file with the previously given filename
                outfile = fopen(filename, "w");
                fileopen = true;

                // Write current buffer into the new jpeg
                fwrite(buffer, BLOCK, 1, outfile);

                // Increment counter for next JPEG
                counter_JPEGs++;
            }
        }

        // Else (= if first 4 bytes do not signal the start of a new file) then just keep adding blocks to current jpeg
        else
        {
            // only write if there is an open file!
            if (fileopen == true)
            {
                fwrite(buffer, BLOCK, 1, outfile);
            }
        }
    }

    // Once arrived here- we have stopped "do", so size of buffer is less than 512. We should close the last file
    fclose(outfile);
    fclose(input);
}